function [vTraceStat,vMaxEigStat,vEigvals, lTraceRank, lMaxEigRank, vCritTrace,vCritMaxEig,vBeta,vAlpha,vEigvalsXcoint,vXcritvals,vAlpha_xyTeststats, vAlpha_xy] = uPSS(vYseries,vXseries,lLagOrder,lCase,lSigniLevel,bBetaAlpha,bExoDiagnostics,bDisplay,bREF_beta_alpha)
%function [vTraceStat,vMaxEigStat,vEigvals, lTraceRank, lMaxEigRank, vCritTrace,vCritMaxEig,vBeta,vAlpha,vEigvalsXcoint,vXcritvals,vAlpha_xyTeststats, vAlpha_xy] = 
%           uPSS(vYseries,vXseries,lLagOrder,lCase,lSigniLevel,bBetaAlpha,bExoDiagnostics,bDisplay,bREF_beta_alpha)
%
%   based on: Pesaran, Shin, Smith (2000), Journal of Econometrics
%   has been tested under MATLAB versions 6 and 7
%________________________________________________________________________________________________________
% INPUTS:
% 1) vYseries (required): matrix with series in columns. Alternatively a stucture Y with Y.data being the data matrix, 
%       and Y.name being a horizontal cell string vector containing the series names.
% 2) vXseries (optional/required): As vYseries. If you don't want to include vXseries, type any scalar or string.
% 3) lLagOrder(required): positive integer. LAg order in LEvels. CAUTION: lag order in the diffs is lLagOrder-1 !!
% 4) lCase (optional, default=5): Positive integer 1 to 5. For the case numbers I-V (see CASES below); if you 
%       provide none or type 'nan', Case V is selected
% 5) lSigniLevel (optional, default=1): Integer. Significance level, type '1', '95' or '5' or 'nan' for the 5% significance level
%       critical values,  type '2', '90' or '10' for the 10% significance level
%       Type '0' resp. 'False' if you do not want the critical values to be retrieved (this will slightly increase calculation speed). However 
%       the critical values for the Exogeneity Diagnostic tests will not be not affected by this choice.
% 6) bBetaAlpha (optional,default=TRUE): Boolean. Type '0' resp. 'False' if you do not want to have Betas and Alphas 
%       computed (see OUTPUT). This will slighltly increase calulation speed;
% 7) bExoDiagnostics (optional, default=TRUE): Boolean. Type '0' resp. 'False' if you do not want the Exognenity Diagnostics tests 
%       (see DIAGNOSTICS below) to be computed (This will increase calulation speed). Not including vXseries sets bExoDiagnostics=FALSE.
% 8) bDisplay (optional, Default=TRUE): Boolean. Type '0' resp. 'False' if you do not want the test results to be displayed in MATLAB.
%       Turning bDisplay off will significantly increase calulation speed. Turning bDisplay on helps you with interpretation,
%       in particluar concerning the exogeneity dioagnostics.
% 9) bREF_beta_alpha (optional, Default=TRUE): Boolean. TRUE will return vBeta and vbAlpha in reduced rank form; FALSE in their 
%       'unnormalised' form.
%________________________________________________________________________________________________________
%OUTPUTS:
%1) vTraceStat: nx1 column vector. Contains the trace statistic, i.e. the partial sums of vMaxEigStat
%2) vMaxEigStat: nx1 column vector. Contains the maximum eigenvalue statistic, i.e. -(T-p)*log(ones(n,1)-vEigvals)
%3) vEigvals: nx1 column vector. Contains the eigenvalues of the generalized eigenvalue problem as in Pesaran et al. (2000), p.303,
%       corresponding to Johansen eigenvalues
%4) lTraceRank: integer >=0 or -1. Contains the minimal rank not rejected by the Trace test (only if lSigniLevel>0).
%       If lSigniLevel=0, then lTraceRank=-1
%5) lMaxEigRank: integer >=0 or -1. Contains the minimal rank not rejected by the Maximum Eigenvalue test (only if 
%       lSigniLevel>0). If lSigniLevel=0, then lMaxEigRank=-1
%6) vCritTrace: nx1 column vector. Contains the 5% or 10% critical values for the Trace test according to input lSigniLevel (see above).
%       If lSigniLevel=0, then vCritTrace is a zero vector
%7) vCritMaxEig: nx1 column vector. Contains the 5% or 10% critical values for the Maximum eigenvalue test according
%       to input lSigniLevel (see above). If lSigniLevel=0, then vCritTrace is a zero vector.
%8) vBeta: cell array with n entries. vBeta{r} is a rxn dimensional matrix, whose rows are the cointegration vectors.
%       if input bBetaAlpha=0, then vBeta=-1
%9) vAlpha: cell array with n entries. vAlpha{r} is a nxr dimensional matrix, its i-th column contains the adjustment
%       coefficients for the i-th cointegrating vector. If input bBetaAlpha=0, then vAlpha=-1
%10) vEigvalsXcoint: kx1 vector. Generalised Eigenvalues for the diagnostic tests of X cointegrated/Stationary (see below)
%       if input bExoDiagnostics=0, then vEigvalsXcoint=0.
%11) vXcritvals: 2x1 vector. Its first entry contains the critical value for the hypo X is I(1), second entry 
%       contains the critical value for the hypo X is cointegrated. If input bExoDiagnostics=0, then vXcritvals=0.
%12) vAlpha_xyTeststats: cell array with n entries. vAlpha_xyTeststats{r} is a scalar, denoting test stat for whether DeltaX depends on Y(t-1).
%       Under hypo of r coint equations it is chi2-distributed. If input bExoDiagnostics=0, then vAlpha_xyTeststats=0.
%13) vAlpha_xy: cell array with n entries. vAlpha_xy{r} is a kxr matrix, whose i-th column contains X's adjustment coefficients
%       for the i-th cointegrating equation. If input bExoDiagnostics=0, then vAlpha_xyTeststats=0.
%________________________________________________________________________________________________________
% EXAMPLE:
%   uPSS(cumsum(normrnd(0,1,100,2)),cumsum(normrnd(0,1,100,1)),2,5)
%________________________________________________________________________________________________________
% OUTLINE
%   This function is based on: Pesaran, Shin, Smith (2000), Journal of Econometrics
%   This function provides the Pesaran/Shin/Smith (2000) trace and Max. Eigenvalue statistics for cointegration of 
%   the n variables in Y, where the k exogenous I(1) variables X enter the cointegration equation.
%   Thus if k=0, then this function provides the Johansen (1991) cointegration test, which applies a 
%   generalized a generalisedeigenvalue problem.
%   Similar to Johansen there are 5 different cases concerning the assumed deterministics (Cases III & V differ slightly):
%
%+ CASES: as in Pesaran/Shin/Smith (2000)
%   1) Case I (no intercepts, no trends)
%   2) Case II (intercept in Coint Equation, no trends)
%   3) Case III (intercept in Coint Equation and VAR, no trends)
%   4) Case IV (intercept in Coint Equation and VAR, trend in Coint Equation)'
%   5) Case V (intercept and trend in Coint Equation as well as VAR)'
%
%+ DIAGNOSTICS
%   The exogenous variables X must be (a) not cointegrated among themseleves and hence (b) not stationary, 
%   and (c) they must not depend on the lagged cointegration relationship with Y.
%   If you put bExoDiagnostics=1/TRUE, then this procedure will provide tests for all these 3 restrictions: 
%   Test stat (a) is: -(T-p) log(1-the first element of vEigvalsXcoint), the critical value is the first element of vXcritvals
%   Test stat (b) is: -(T-p) sum(log(1-vEigvalsXcoint)), the critical value is the second element of vXcritvals
%   Test stat (c) is given by vAlpha_xyTeststats{r} for each hypothesised rank r and is chi2-distributed.
%________________________________________________________________________________________________________
%   
%   Done for first year paper at the Institute of Advanced Studies, Vienna, by
%   STEFAN ZEUGNER 2006 - www.zeugner.eu
%________________________________________________________________________________________________________




%__________________ INPUT CHECKS ______________________________________________________  
    
   
    error(nargchk(2, 9, nargin));
    if nargin<3;error('Please provide a Lag Order (in levels), at least 1');end    
    if nargin<4;lCase=5;end    
    if nargin<5;lSigniLevel=1;end    
    if nargin<6;bBetaAlpha=1;end        
    if nargin<7;bExoDiagnostics=1;end    
    if nargin<8;bDisplay=1;end    
    if nargin<9;bREF_beta_alpha=1;end    

    
    if lSigniLevel==95|lSigniLevel==0.95|lSigniLevel==5|lSigniLevel==0.05;lSigniLevel=1;end
    if lSigniLevel==90|lSigniLevel==0.90|lSigniLevel==10|lSigniLevel==0.10;lSigniLevel=2;end
    if lSigniLevel==0;bCritVals=0;lSigniLevel=1;else;bCritVals=1;end
        
    if isempty(vXseries);vXseries=1;end
    if isempty(lCase)|isnan(lCase);lCase=5;end
    if ~ismember(lSigniLevel,[1,2]);lSigniLevel=1;end    
    if ~ismember(bBetaAlpha,[0,1]);bBetaAlpha=1;end        
    if ~ismember(bExoDiagnostics,[0,1]);bExoDiagnostics=1;end    
    if ~ismember(bDisplay,[0,1]);bDisplay=1;end    
    if ~ismember(bREF_beta_alpha,[0,1]);bREF_beta_alpha=1;end    

    if floor(lLagOrder)~=lLagOrder|lLagOrder<1;
        error('Please provide a postive integer as lag order');
    end
    if ~ismember(lCase,1:5);
        error('Please proivide a Case number between 1 and 5');
    end    
    if isstruct(vYseries);
        try sYnames=vYseries.name;catch,error('Please provide vYseries either as a matrix, or as a structure with fields ''data'' and ''name''');end
        try vYseries=vYseries.data;catch,error('Please provide vYseries either as a matrix, or as a structure with field ''data'' in addition to field ''name''');end
    end    
    if isstruct(vXseries);
        try sXnames=vXseries.name;catch,error('Please provide vXseries either as a matrix, or as a structure with fields ''data'' and ''name''');end
        try vXseries=vXseries.data;catch,error('Please provide vXseries either as a matrix, or as a structure with field ''data'' in addition to field ''name''');end
    end    
    
    [lTdata,n] = size(vYseries);
    [lTdata2,k] = size(vXseries);
    
    
    if lTdata2~=lTdata;k=0;end
    if n>12;error('Maximum number of columns in Y is 12!');end
    if k>5;error('Maximum number of columns in X is 5!');end    
    if k==0;bExoDiagnostics=0;end    
    lTraceRank=-1;lMaxEigRank=-1;vBeta=-1;vAlpha=-1;
    if bExoDiagnostics==0;vLogEigsX=-1;vAlpha_xyTeststats=-1;vAlpha_xy=-1;end
    
    if bBetaAlpha==0;bREF_beta_alpha=0;end
    if lLagOrder>lTdata;
        error('Please decrease your lag order!');
    end
    
%__________________ STATISTICS ______________________________________________________      
    
    %get statistics from calculation function
    [vEigvals,vEigvecsTranspo,vBetaUnNormedTranspo,T,lNewn,vAlphaUnnormed,vAlpha_xyTranspo,vAlpha_xyTeststats,vEigvalsXcoint] = fnc_psscompute(vYseries,vXseries,lLagOrder,lCase,n,k,lTdata,bExoDiagnostics,bBetaAlpha);

    %transform to the two test statistics
    vMaxEigStat = -T*log(ones(n,1)-vEigvals);
    for r=1:n;
        vTraceStat(r,1)=sum(vMaxEigStat(r:n));
    end
    
    
    if bCritVals; %get critical values and calc rank recommendation
        vCrits=(fnc_psscritval(k+1,(13-n:12),lSigniLevel,lCase,(1:2))); %get critvals from special function
            vCritTrace=squeeze(vCrits(1,:,1,1,1))';
            vCritMaxEig=squeeze(vCrits(1,:,1,1,2))';        
            
        lTraceRank=0;lMaxEigRank=0;
        for r=1:n
            if vTraceStat(r,1)>vCritTrace(r);lTraceRank=r;end
            if vMaxEigStat(r,1)>vCritMaxEig(r);lMaxEigRank=r;end            
        end
        
    else
        vCritTrace=zeros(n,1);vCritMaxEig=zeros(n,1);
    end
    
    if bBetaAlpha; %only if requested, do Coint vec & Adjust Coefs
        clear vBeta;clear vAlpha;
        if bREF_beta_alpha; %convert beta, alpha to reduced row echelon form
            for r=1:n;
                vBeta{r}=rref(vBetaUnNormedTranspo(1:r,:)); %=inv(vBeta(1:r,1:r)*vBeta
                vAlpha{r}=vAlphaUnnormed{r}*vEigvecsTranspo(1:r,1:r)';
            end
        else %let beta in its usaual, left eigenvector, normalisation
            for r=1:n;
                vBeta{r}=vBetaUnNormedTranspo(1:r,:);
            end
            vAlpha=vAlphaUnnormed;
        end
    end
    
        if bExoDiagnostics;
            
            vXcritvals = squeeze(fnc_psscritval(1,(13-k),lSigniLevel,lCase,(1:2))); %get vcritvals for the stationarity & coint tests on X
            
            vLogEigsX=-T*log(ones(k,1)-vEigvalsXcoint); %prepare for trace & maxeig stat
            if bREF_beta_alpha;
                for r=1:n;
                    vAlpha_xy{r}=vAlpha_xyTranspo{r}'*vEigvecsTranspo(1:r,1:r); %transform to accomdate the red row echelon form of beta
                end
            else
                for r=1:n;                
                    vAlpha_xy{r}=vAlpha_xyTranspo{r}';
                end
            end
        else
            vXcritvals=zeros(k,1);
        end

%__________________ DISPLAY ______________________________________________________          
    if bDisplay;
        
        %if no names have been provide, create ones
        if k>0;
            if ~exist('sXnames');
                sXnames=cellstr(strcat('X',num2str((1:k)')))';
            elseif k==1
                sXnames=cellstr(sXnames);
            end
        else
            sXnames=1;
        end
        if ~exist('sYnames');
            sYnames=cellstr(strcat('Y',num2str((1:n)')))';
        elseif n==1
            sYnames=cellstr(sYnames);
        end
        
        % do the display in sub function
        bSuxess=fnc_DisplayPSSresults(n,k,lLagOrder,lCase,lSigniLevel,bBetaAlpha,bExoDiagnostics,bDisplay,bREF_beta_alpha,bCritVals,T,vEigvecsTranspo,vLogEigsX,vAlpha_xyTeststats,vAlpha_xy,vTraceStat,vMaxEigStat,vEigvals, lTraceRank, lMaxEigRank, vCritTrace,vCritMaxEig,vBeta,vAlpha,sXnames,sYnames,vXcritvals);
    end
    
    
%#####################################################################################################    

function [vEigvalsFinal,vEigvecsFinalTranspo,vBeta,T,lNewn,vAlpha,valpha_xy,vTest_alpha_xy,vEigvalsXcoint] = fnc_psscompute(vY,vX,p,lCase,n,k,lTdata,bExoDiag,bBetaAlpha)
%function [vEigvalsFinal,vEigvecsFinalTranspo,vBeta,T,lNewn,vAlpha,valpha_xy,vTest_alpha_xy,vEigvalsXcoint] = fnc_psscompute(vY,vX,p,lCase,n,k,lTdata,bExoDiag,bBetaAlpha)
%
%   do spuit de musi!

%__________________ PREPARE CONCENTRATING OUT ______________________________________________________  
    %construct Z
    T = lTdata-p;
    if k>0;
    	vZ = cat(2,vY,vX);
    else
        vZ = vY;
    end

    %get diffs in adjusted sample T        
    vDZ = diff(vZ);            
    if p>1;
      vDZstar = vDZ(p-1:lTdata-2,:);
    else
      vDZstar = zeros(T,1); % if no lagged diffs included, we have to fill vDZstar with sth. anyway, corrected for later
    end

    %construct Delta Z_{-}^*
	for i=2:p-1;
        vDZstar = cat(2,vDZstar,vDZ(p-i:lTdata-i-1,:));
    end
    
    %construct Delta Y
    vDY = vDZ(p:(lTdata-1),1:n);

    vOnes=ones(T,1);
    vTrend=(1:1:T)';
    
    %Do cases 
    switch lCase
        case 1
            vZstar=vZ(p:lTdata-1,:);
            vDesign_woX = vDZstar;
            lNewn=n;
        case 2
            vZstar=cat(2,vZ(p:lTdata-1,:),vOnes);
            vDesign_woX = vDZstar;
            lNewn=n+1;
        case 3
            vZstar=vZ(p:lTdata-1,:);
            vDesign_woX = cat(2,vOnes,vDZstar);
            lNewn=n;                        
        case 4
            vZstar=cat(2,vZ(p:lTdata-1,:),vTrend);
            vDesign_woX = cat(2,vOnes,vDZstar);
            lNewn=n+1;
        case 5
            vZstar=vZ(p:lTdata-1,:);
            vDesign_woX = cat(2,vOnes,vTrend,vDZstar);          
            lNewn=n;            
        otherwise
            
    end
    
        % add 
        if k>0;
            vDX = vDZ(p:(lTdata-1),n+1:n+k);
            vDesign = cat(2,vDX,vDesign_woX);
        else
            vDesign = vDesign_woX;
        end
     
        if p==1;vDesign=vDesign(:,1:size(vDesign,2)-1);end %correcttion for the thing done 50 lines above
        
     %Make Projector matrix by using singular value decomposition
            [vU,vS,vV]=svd(vDesign);
            lNbCols=size(vV,2);
            vUproject=vU(:,1:lNbCols);
            vProjector=vUproject*vUproject';
            %Calculate residuals
            vZhat = vZstar - vProjector*vZstar;
            vDYhat = vDY - vProjector*vDY;

            
%__________________ DO GENERALISED EIGENVALUES ______________________________________________________              
    %construct Syy Szz Syz - decompose to have as few matrix multiplications kxT*Txk as possible (there Matlab gets really slow)
    vSyy=vDYhat'*vDYhat/T;
    vSyz=vDYhat'*vZhat/T  ;  
    vSzz=vZhat'*vZhat/T;
    vSall=[vSyy, vSyz; vSyz',vSzz];
    %get generic eigenvalue probelm solved
    [vEigvalsFinal,vEigvecsFinalTranspo]=fnc_geneig(vSall,lNewn+k,n);
    
    % add const and trend to cointegrating equation in unrestricted cases, just as eviews does
    if bBetaAlpha;
        if lCase==3|lCase==4;
               vBeta=[vEigvecsFinalTranspo,-mean(vZstar*vEigvecsFinalTranspo')'];
           elseif lCase==5;
               vBeta=[vEigvecsFinalTranspo,-(2/T/(T+1)/(T-1)*[(T+1)*(2*T+1),-3*(T+1);-3*(T+1),6]*[vOnes,vTrend]'*(vZstar*vEigvecsFinalTranspo'))'];
               %the cool thing is the inverse of [vOnes,vTrend]'*[vOnes,vTrend]
           else
               vBeta=vEigvecsFinalTranspo;
        end
        for r=1:n
            vAlpha{r}=vSyz*(vEigvecsFinalTranspo(1:r,:))';
        end
    else
        vBeta=0;vAlpha=0;
    end;
    
    
    %__________________ DO X-DIAGNOSTICS ______________________________________________________  
    if bExoDiag;
        %preparation for alpha_xy
        vDX = vDZ(p:(lTdata-1),n+1:n+k);
        vCointResid=vZstar*vEigvecsFinalTranspo';
        if p>1|lCase>2;
            if p==1, %necessary if p=1, but we require detrending/demeaning
                vXdesign=vDesign_woX(:,1:size(vDesign_woX,2)-1)    ;
            else
                vXdesign=vDesign_woX; 
            end
             [vU,vS,vV]=svd(vXdesign);
             lNbCols=size(vV,2);
             vUproject=vU(:,1:lNbCols);
             vProjector=vUproject*vUproject';
        else    %necessary in this spec.case
                vProjector=0;
        end
            vExRestr=vDX-vProjector*vDX; %=vDXhat
            vCointResidhat=vCointResid-vProjector*vCointResid;
            
    %preparation for X coint test
        vXstar=vZstar(:,n+1:lNewn+k);
        vDXhat=vExRestr;
        if p>1;
            vXhatStar=vXstar-vProjector*vXstar;        
        else
            vXhatStar=vXstar;
        end

    %Doing alpha_xy    
        for r=1:n;
            [vUc,vSc,vVc]=svd(vCointResidhat(:,1:r));
            valpha_xy{r}=vVc*diag(diag(vSc(1:r,1:r)).^(-1))*vUc(:,1:r)'*vExRestr;
            vExFree=vExRestr-vCointResidhat(:,1:r)*valpha_xy{r};
            vTest_alpha_xy{r}=T*(log(det(vExRestr'*vExRestr))-log(det(vExFree'*vExFree)));
        end

    %Doing X Cointegration test    
        vS00=vDXhat'*vDXhat/T;
        vS01=vDXhat'*vXhatStar/T  ;  
        vS11=vXhatStar'*vXhatStar/T;
        vSXcoint=[vS00, vS01; vS01',vS11];
        [vEigvalsXcoint,vEigvecsXcointTranspo]=fnc_geneig(vSXcoint,length(vS11),k);
    else
        valpha_xy=0;vTest_alpha_xy=0;,vEigvalsXcoint=0;
    end
    
    
    
%#####################################################################################################    
    function [vEigvalues,vEigvectorsTrans]=fnc_geneig(vSZall,m,n)
        %decompose for numerical efiicieny and transform problem into symm. one
        vL=chol(vSZall)'; 
        vLz=chol(vSZall(n+1:m+n,n+1:m+n))';
    
        vL21=vL((n+1):(n+m),1:n);
        vLzInv=vLz\eye(m);


        [vEigvecsPre, vEigvals]=eig(vLzInv*vL21*vL21'*vLzInv'); %get eigens of symm matrix
        vLeftEigvecs=vLzInv'*real(vEigvecsPre); %sortrows can't deal with complex numbers
        vEigmValsVecs=sortrows(cat(2,-real(diag(vEigvals)),vLeftEigvecs')); %sort by eigenvalue, descending
        %get the sorted eigvals & eigvecs; all of order >n are 0
        vEigvalues = -vEigmValsVecs(1:n,1); 
        vEigvectorsTrans = vEigmValsVecs(1:n,2:(m+1));

        
%#####################################################################################################    

function soutchar=fnc_DisplayCellStr(csIn,lBlanksbetween,lRowsbetween,lLeadingBlanks,cColAlign,cRowAlign)
%This function converts a cellstring array into a char, by using the layout
%   options given in the optional inputs: i.e. it makes a 'nice table' out of
%   a cell string
%    if you don't want to specify an optional input, put 'nan' as in the
%    example below
%   This function is just a copy of my function fncCstr2format
%____________________________________________________
%OUTPUTS
%   soutchar: char string containing a nicely formatted output
%____________________________________________________
%INPUTS
    %csIn (required) is a cell array of strings
    %lBlanksbetween (optional) is the number of blanks between columns
    %   (default=1)
    %lRowsbetween (optional) is the number of empty rows between the rows to be
    %   displayed (default=0) (currently decactivated)
    %lLeadingBlanks (optional) is the number of blanks to be put before the out
    %   (default=0)
    %cColAlign (optional) is a double array with alignment of columns (-1=left, 0=center, 1=right)
    %   (default=vector of -1)
    %cRowAlign (optional) is a double array with alignment of rows (-1=left, 0=center,%1=right) 
    %   (default=vector of -1)
    %   cRowAlign (optional) overrides the formatting of cColAlign; if you want cRowAlign not
    %   to change the cColAlign alignment for a specific row, put in another
    %   number than -1, 0 or 1 in the entry for that specifc row
%____________________________________________________    %
%EXAMPLE:
%   cellstringinput={'abc','defg';'hijkl','mn'}
%   formatedOutput=fnc_DisplayCellStr(cellstringinput,4,nan,10,[1;0],[-1;9])
%______________________________________________________________________
%   Martin Feldkircher & Stefan Zeugner 2006




    error(nargchk(1, 6, nargin))
    
        if nargin<6,cRowAlign=nan;end
        if nargin<5,cColAlign=nan;end
        if nargin<4,lLeadingBlanks=nan;end
        if nargin<3,lRowsbetween=nan;end
        if nargin<2,lBlanksbetween=nan;end

        if isnan(lBlanksbetween), lBlanksbetween=1;end
        if isnan(lRowsbetween), lRowsbetween=0;end
        if isnan(lLeadingBlanks), lLeadingBlanks=0;end
        if isnan(cColAlign), cColAlign=-ones(size(csIn,2),1); end
        if isnan(cRowAlign), cRowAlign=2*ones(size(csIn,1),1); end        

for i=1:size(csIn,2)
   scont{i}=char(csIn{:,i});
end

for i=1:size(cColAlign,1)
    if ismember(cColAlign(i,1),[-1,0,1])
        scont{i}=strjust(scont{i},convalign(cColAlign(i,1)));
    end
end

for i=1:size(cRowAlign,1)
    if ismember(cRowAlign(i,1),[-1,0,1])
        for j=1:length(scont)
            scont{j}(i,:)=strjust(scont{j}(i,:),convalign(cRowAlign(i,1)));
        end
    end
end


scoldelim=char(zeros(size(csIn,1),lBlanksbetween));
stempcont=[char(zeros(size(csIn,1),lLeadingBlanks)) scont{1}];
for i=2:size(csIn,2)
   stempcont=[stempcont scoldelim scont{i}];
end


%if lRowsbetween>0
%    shihi=''
%     for i=1:size(stempcont,1)
%         strcat( shihi,' \n ', stempcont(i,:), repmat(' \n ' ,1,lRowsbetween))
%        shihi=sprintf(strcat( shihi,' \n ', stempcont(i,:), repmat(' \n ' ,1,lRowsbetween)));
%     end
% end
% disp(shihi)

soutchar=stempcont;

function sOut=convalign(dIn)
switch dIn
    case -1
        sOut='left';
    case 0
        sOut='center';
    case 1
        sOut='right';
end



%#####################################################################################################    

function csOut=fnc_c2cstr(din,sformat,bJustifyDelimiter)
% function csOut=c2cstr(din,sformat,bJustifyDelimiter)
%
% This function converts a double array into a nicely-looking cell array of
%   strings
% This function is just a copy of my function c2cstr
%_______________________________________
%OUTPUTS:
% csOut: cellstring array of strings
%_______________________________________
%INPUTS:
% din (required): double matrix 
% sformat (optional): format and precision of numbers to be displayed (see
%       MATLAB help: 'fprintf'). Default='%-5G'
% bJustifyDelimiter (optional): If true, then the the numbers are justified
% (aligned) along the decimal delimiter '.'
%_______________________________________
% This function is intended to be used only by other functions, so there
% are no input checks
%_______________________________________
%   Martin Feldkircher & Stefan Zeugner 2006

    if str2num(strtok(version,'.'))<7,csOut=cellstr(num2str(din));return;end
    
    if nargin<2,sformat='%-5G';end
    if isnan(sformat)|isempty(sformat),sformat='%-5G';end
    if nargin<3,bJustifyDelimiter=1;end
    csTemp = cellstr(num2str(din(:,1),sformat));
    if bJustifyDelimiter
        [sleft,sright]=strtok(csTemp,'.');
        csOut=strcat(cellstr(strjust(char(sleft),'right')),sright);
    else
        csOut=csTemp;
    end
    

%#####################################################################################################        
    
function bSuxess=fnc_DisplayPSSresults(n,k,lLagOrder,lCase,lSigniLevel,bBetaAlpha,bExoDiagnostics,bDisplay,bREF_beta_alpha,bCritVals,T,vEigvecsTranspo,vLogEigsX,vAlpha_xyTeststats,vAlpha_xy,vTraceStat,vMaxEigStat,vEigvals, lTraceRank, lMaxEigRank, vCritTrace,vCritMaxEig,vBeta,vAlpha,sXnames,sYnames,vXcritvals)
        bSuxess=0;
        
        sTrendName={'TREND'};
        sConstName={'CONST'};
        
        
        lLeadBlanks=4;
        
        %__________________ TRACE & MAXEIG ______________________________________________________  
        switch lCase
            case 1
                sCaseDesc='Case I (no intercepts, no trends)';
            case 2
                sCaseDesc='Case II (intercept in Coint Equation, no trends)';
            case 3
                sCaseDesc='Case III (intercept in Coint Equation and VAR, no trends)';     
            case 4
                sCaseDesc='Case IV (intercept in Coint Equation and VAR, trend in Coint Equation)';
            case 5
                sCaseDesc='Case V (intercept and trend in Coint Equation as well as VAR)';
        end

        
        if lSigniLevel==2,sCritval='10%';else;sCritval='5%';end
        csTraceDisplay=cat(2,fnc_c2cstr((0:1:n-1)'),fnc_c2cstr(vEigvals), fnc_c2cstr(vTraceStat),fnc_c2cstr(vCritTrace));
        csTraceDisplay=cat(1,{'Nb. CEs','Eigenvalue','Trace Stat',sprintf('%s Crit. Value',sCritval)},csTraceDisplay);
        csMaxEigDisplay=cat(2,fnc_c2cstr((0:1:n-1)'),fnc_c2cstr(vEigvals), fnc_c2cstr(vMaxEigStat),fnc_c2cstr(vCritMaxEig));
        csMaxEigDisplay=cat(1,{'Nb. CEs','Eigenvalue','Max.Eigenvalue',sprintf('%s Crit. Value',sCritval)},csMaxEigDisplay);        
        sStatDisplay=fnc_DisplayCellStr(cat(1,csTraceDisplay,csMaxEigDisplay),2,0,lLeadBlanks,0);
        if bCritVals;
            sTraceImpli=sprintf('%sTrace test indicates %g CEs at the %s significance level.',repmat(' ',1,lLeadBlanks),lTraceRank,sCritval);
            sMaxEigImpli=sprintf('%sMax.Eigenvalue test indicates %g CEs at the %s significance level.',repmat(' ',1,lLeadBlanks),lMaxEigRank,sCritval); 
        end
        
        %__________________ COINT VECS ______________________________________________________          
        if k>0;csEigvec=cat(1,sYnames',sXnames');else;csEigvec=sYnames';end
        if lCase==2;csEigvec=cat(1,csEigvec,sConstName);end
        if lCase==4;csEigvec=cat(1,csEigvec,sTrendName);end
        for r=1:n
            csEigvec=cat(2,csEigvec,fnc_c2cstr(vEigvecsTranspo(r,:)'));    
        end
        sEigvec=fnc_DisplayCellStr(csEigvec',2,0,lLeadBlanks+2,0);
        
        sDelimLine=sprintf('%s%s',repmat(' ',1,lLeadBlanks-1),repmat('-',1,size(sEigvec,2)-lLeadBlanks+2));
        sDelimLine2=sprintf('%s%s',repmat(' ',1,lLeadBlanks+2),repmat('=',1,size(sEigvec,2)-lLeadBlanks));        
        sDelimLine3=sprintf('%s%s',repmat(' ',1,lLeadBlanks+4),repmat('*',1,size(sEigvec,2)-lLeadBlanks));
        sDelimLine0=sprintf('%s%s',repmat(' ',1,lLeadBlanks-1),repmat('-',1,size(sStatDisplay,2)-lLeadBlanks+2));        
        
        %__________________ DISPLAY TRACE, MAXEIG & EIGVECS ______________________________________________________  
        disp(sprintf('\n %s PESARAN-SHIN-SMITH COINTEGRATION TEST %s \n',repmat('#',1,10),repmat('#',1,10)));
        disp(sprintf('%sAdjusted Sample Size: %g',repmat(' ',1,lLeadBlanks),T));
        disp(sprintf('%sLags in Levels: %g (i.e. %g in first differences)',repmat(' ',1,lLeadBlanks),lLagOrder,lLagOrder-1));
        disp(sprintf('%sTrend assumption: %s\n',repmat(' ',1,lLeadBlanks),sCaseDesc));

        disp(sDelimLine0);
        disp(sStatDisplay(1,:));
        disp(sDelimLine0);
        disp(sStatDisplay(2:n+1,:));        
        disp(sDelimLine0);        
        if bCritVals;disp(sTraceImpli);end
        disp(sprintf('\n%s',sDelimLine0));        
        disp(sStatDisplay(n+2,:));
        disp(sDelimLine0);
        disp(sStatDisplay(n+3:2*n+2,:));
        disp(sDelimLine0);
        if bCritVals;disp(sMaxEigImpli);end        
        
        disp(sprintf('\n\n%sUnrestricted Cointegration Coefficients:',repmat(' ',1,lLeadBlanks+1)));
        disp(sDelimLine);        
        disp(sEigvec);        
        disp(sDelimLine);        
        

        if bExoDiagnostics;
        %__________________ EXO COINT & I(1) TEST ______________________________________________________          
            csXdiagStat={'Hypothesis','Test Statistic',sprintf('%s Crit. Value',sCritval);'X is I(1)',num2str(sum(vLogEigsX)),num2str(vXcritvals(1))};
            if k>1;csXdiagStat=cat(1,csXdiagStat,{'X not cointegrated',num2str(vLogEigsX(1,1)),num2str(vXcritvals(2))});end
            sXdiagStat=fnc_DisplayCellStr(csXdiagStat,2,0,lLeadBlanks,-1);
            if sum(vLogEigsX)>vXcritvals(1);sExoImpli1='X SEEMS TO BE I(0)!';else;sExoImpli1='Null of X~I(1) cannot be rejected.';end
            if vLogEigsX(1,1)>vXcritvals(2);sExoImpli2='X SEEMS TO BE COINTEGRATED!';else;sExoImpli2='Null of No Cointegration cannot be rejected.';end
                       
            disp(sprintf('\n\n%s%s X-DIAGNOSTICS %s \n',repmat(' ',1,lLeadBlanks-2),repmat('#',1,8),repmat('#',1,20)));
            disp(sprintf('%sTest for Stationarity/Cointegration of X',repmat(' ',1,lLeadBlanks)));
            disp(sDelimLine);
            disp(sXdiagStat(1,:));
            disp(sDelimLine);
            disp(sXdiagStat(2:size(sXdiagStat,1),:));
            disp(sDelimLine);
            disp(sprintf('%s%s',repmat(' ',1,lLeadBlanks+2),sExoImpli1));
            if k>1;disp(sprintf('%s%s',repmat(' ',1,lLeadBlanks+2),sExoImpli2));end


        %__________________ EXO ALPHA_xy TEST ______________________________________________________                      
            disp(sprintf('\n\n%sTest whether X is not long-run dependent on Y',repmat(' ',1,lLeadBlanks)));
            csDispExoTeststat={'Nb. CEs','Alpha_xy LR stat',sprintf('Prob. Value')};
            for r=1:n;
               csDispExoTeststat=cat(1,csDispExoTeststat,{num2str(r), num2str(vAlpha_xyTeststats{r}),num2str(1-chi2cdf(vAlpha_xyTeststats{r},k*r))});
            end 
            sDispExoTeststat=fnc_DisplayCellStr(csDispExoTeststat,2,0,lLeadBlanks,0,0);
            if lSigniLevel==2;dSignialpha=0.1;else;dSignialpha=0.05;end
            if bCritVals;
                if lTraceRank>0;
                    if 1-chi2cdf(vAlpha_xyTeststats{lTraceRank},k*lTraceRank)<dSignialpha;
                        sAlpha_xyImpli1=sprintf('X depends on levels of Y - under Hypothesis of %g CEs (as recommended by the trace test above)!',lTraceRank);
                    else
                        sAlpha_xyImpli1=sprintf('Null that X is indep. of Y levels not rejected -  under Hypothesis of %g CEs (as recommended by trace test above), !',lTraceRank);;
                    end
                else;sAlpha_xyImpli1='';end
                if lTraceRank~=lMaxEigRank;
                    if 1-chi2cdf(vAlpha_xyTeststats{lMaxEigRank},k*lMaxEigRank)<dSignialpha;
                        sAlpha_xyImpli2=sprintf('Under Hypothesis of %g CEs (recommended by the max.eig. test above), X depends on levels of Y!',lMaxEigRank);
                    else
                        sAlpha_xyImpli2='';
                    end
                end
            else
                sAlpha_xyImpli1=''; sAlpha_xyImpli2='';
            end
            
            disp(sDelimLine);            
            disp(sDispExoTeststat(1,:));
            disp(sDelimLine);            
            disp(sDispExoTeststat(2:size(sDispExoTeststat,1),:));
            disp(sDelimLine);
            disp(sprintf('%s%s',repmat(' ',1,lLeadBlanks+2),sAlpha_xyImpli1));
            if bCritVals;if lTraceRank~=lMaxEigRank;disp(sprintf('%s%s',repmat(' ',1,lLeadBlanks+2),sAlpha_xyImpli2));end;end;


            disp(sprintf('\n\n%sCorresponding Alpha_xys',repmat(' ',1,lLeadBlanks)));               
            for r=1:n;
                sAlpha_xyStatr=cat(1,{'CE Nb.'},fnc_c2cstr((1:r)'));
                for i=1:k
                    sAlpha_xyStatr=cat(2,sAlpha_xyStatr,cat(1,{sXnames{i}},fnc_c2cstr(vAlpha_xy{r}(i,1:r)')));
                end
                disp(sDelimLine2);                                
                disp(sprintf('%sAlpha_xy under %g CEs:',repmat(' ',1,lLeadBlanks+3),r));               
                disp(fnc_DisplayCellStr(sAlpha_xyStatr,2,0,lLeadBlanks+3,0,0));                   
            end
            disp(sDelimLine);
        end
        
                
        if bBetaAlpha 
        %__________________ DISPLAY BETA & ALPHA ______________________________________________________          
            if k>0;sAllNames=cat(1,sYnames',sXnames');else;sAllNames=sYnames';end
            if ismember(lCase,2:5);sAllNames=cat(1,sAllNames,sConstName);end
            if ismember(lCase,4:5);sAllNames=cat(1,sAllNames,sTrendName);end

            disp(sprintf('\n\n%s%s ALPHAS & BETAS %s \n',repmat(' ',1,lLeadBlanks-2),repmat('#',1,8),repmat('#',1,20)));
            for r=1:n-1;
                csBeta=sAllNames;
                csAlpha=sYnames';
                for i=1:r
                    csBeta=cat(2,csBeta,fnc_c2cstr(vBeta{r}(i,:)'));
                    csAlpha=cat(2,csAlpha,fnc_c2cstr(vAlpha{r}(:,i)));
                end;
                disp(sDelimLine2);
                if bREF_beta_alpha;snormed='Normalized ';else;snormed='';end
                disp(sprintf('%s%sCointegration Coefs under %g CEs:',repmat(' ',1,lLeadBlanks+4),snormed,r));               
                disp(fnc_DisplayCellStr(csBeta',2,0,lLeadBlanks+5));
                disp(sDelimLine3);
                disp(sprintf('%sCorresponding Adjustment Coefs under %g CEs:',repmat(' ',1,lLeadBlanks+4),r));               
                disp(fnc_DisplayCellStr(csAlpha',2,0,lLeadBlanks+5));
            end    
            disp(sDelimLine2);
            disp(sDelimLine);
        end    
        disp(sprintf('\n %s',repmat('#',1,60)));
        disp(sprintf('\n%sCompare Pesaran, Shin, Smith (2000), Journal of Econometrics\n%sDone by Stefan Zeugner 2006; www.zeugner.eu\n',repmat(' ',1,lLeadBlanks+2),repmat(' ',1,lLeadBlanks+2)));        
        bSuxess=1;

        
%#####################################################################################################                
        
function vCritsRequested = fnc_psscritval(vKplus1,vN_r,vSigni,vCase,vStat)
		%function vOut = fnc_psscritval(vStat,vCase,vSigni,vN_r,vKplus1)
		%   CAUTION: no usability checks!
		%
		%INPUTS:
		%   vKplus1: double scalar or vector of values 1:6, ie "1" for k=0, "6" for k=5
		%   vN_r: double scalar or vector of values 1:12, ie "1" for n-r=12, "12" for n-r=1
		%   vSigni: double scalar or vector of values 1:2, 1=5%level, 2=10%level
		%   vCase double scalar or vector of values 1:5, 1=Case I to 5=Case V
		%   vStat: double scalar or vector of values 1:2, 1=TraceCritval, 2=MaxEigCritval
		%
		%OUTPUT:
		%   vOut is an array of dimensions in the same order as the input:
		%       (k+1,n-r,0.05signi:0.1signi,Case(I:V),Trace(1):Maxeig(:))
		%   Hint: You may want to apply the "squeeze" function to the output
		%
		% EXAMPLE:
		%   squeeze(fnc_psscritval([1,3],12,2,5,1))
		%   returns Trace critical values for case 5, 10% significance level, n-r=1 and k=0 as well as k=2
		%
		%critarray dimensions: vCritArray(k,13-(n-r),0.05signi:0.1signi,Case(I:V),Trace(1):Maxeig(:))
		
		vCritArray=...
            cat(5,...
		cat(4,...
		cat(3,[304.4,258.7,215.4,175.9,141.2,110.1,83.18,59.33,39.81,24.05,12.36,4.16;...
		329.4,280.5,235.8,195.9,158.2,125.9,96.79,69.87,49.56,32.16,17.79,8.13;...
		353.5,303.1,256.7,214.9,175.3,141.4,110.2,81.45,59.19,39.4,23.37,11.41;...
		378,326,277.4,233.5,192.1,156.7,123.3,92.92,68.7,46.79,28.64,14.19;...
		402,347.9,298.7,252.1,209.3,171.4,136,104.2,77.6,53.72,33.68,17.24;...
		426.2,370.3,318.6,271,225.7,185.6,148.7,115.3,86.38,60.47,38.1,19.76],[295.7,251.2,208,169.8,135.6,105.4,78.47,55.42,36.69,21.46,10.25,3.04;...
		320.6,273,228.8,189.1,152.3,120.1,91.9,66.14,45.95,29.08,15.83,6.49;...
		344.8,295.4,249.3,207.7,168.9,135.5,104.9,77.11,55.07,36.2,20.93,9.52;...
		368.9,317.2,269.6,226.2,185.6,150.4,117.5,87.89,64.13,43.24,25.81,12.21;...
		392.1,338.9,289.9,244.7,202.3,164.7,130.5,99.28,72.93,49.87,30.68,14.91;...
		415.8,360.6,310.1,262.7,218.5,179,143,109.9,81.39,56.55,35.26,17.4]),...
		cat(3,[341.2,291.7,245.7,204,166.1,132.5,102.6,75.98,53.48,34.87,20.18,9.16;...
		365.1,314.2,265.9,223.8,182.7,148,116.3,86.58,62.75,42.4,25.23,12.45;...
		389.4,336.4,287,242.3,199.7,163.1,128.9,97.57,72.15,49.43,30.46,15.27;...
		413.6,358.7,307.2,260.8,216.6,178,141.7,108.6,81.25,56.28,35.46,17.8;...
		437.1,381,327.8,279.5,232.8,192.5,154.4,119.8,90.6,63.1,39.94,20.63;...
		461.5,403.4,348.5,297.9,249.3,206.5,167,130.8,99.37,69.72,44.56,23.02],[332,284.2,238.3,197.7,160.3,127.2,97.87,71.81,49.95,31.93,17.88,7.53;...
		356.4,305.4,259,216.4,176.4,142.2,110.5,82.17,59.07,39.12,22.76,10.5;...
		380.3,328.1,279,234.7,192.9,157,123.5,92.93,67.83,45.89,27.58,13.21;...
		403.8,350.1,299.3,253,209.2,171.5,136,103.6,76.69,52.71,32.28,15.68;...
		427.3,371.6,319.2,271.4,226.1,185.7,148.3,114.4,85.34,59.23,36.84,18.24;...
		450.6,393.4,338.9,289.3,241.8,199.5,160.6,125.2,93.92,65.69,41.33,20.57]),...
		cat(3,[328.5,279.8,235,194.4,157.8,124.6,95.87,70.49,48.88,31.54,17.86,8.07;...
		352.4,302.2,255.5,214.4,174.2,140.5,109.6,81.45,58.63,38.93,23.32,11.47;...
		376.5,324.4,276.5,232.7,191.6,155.9,122.8,92.42,68.06,46.44,28.42,14.35;...
		400.8,347.7,297.2,251.6,208.3,170.7,135.4,103.7,77.21,53.41,33.35,16.9;...
		424.3,369.5,317.4,270.4,224.8,185.6,148.4,114.6,86.42,60.19,38.15,19.73;...
		448.8,391.5,337.7,288.6,241.2,199.8,160.9,125.9,95.14,66.94,42.73,22.19],[319.6,272.7,227.7,187.9,152,119.7,91.4,66.23,45.7,28.78,15.75,6.5;...
		343.5,294.1,248.3,207.1,168.4,134.9,104.4,76.95,54.84,35.88,20.75,9.53;...
		367.5,316.9,268.5,225.3,184.8,149.7,117.3,87.93,63.57,42.67,25.63,12.27;...
		391.1,338.8,288.8,243.7,201.3,164.2,130,98.45,72.69,49.56,30.37,14.76;...
		415,360.1,309.1,262.1,217.7,178.3,142.1,109.3,81.4,56.08,35.06,17.39;...
		438,381.8,328.9,280.2,233.6,192.3,154.6,120.4,89.9,62.65,39.59,19.63]),...
		cat(3,[364.8,314.1,265.8,222.6,183,147.3,115.9,87.17,63,42.34,25.77,12.39;...
		389.2,335.5,286.8,242.4,199.1,163,128.8,97.83,72.1,49.36,30.77,15.44;...
		413.5,358,307,260.4,215.8,177.8,141.7,108.9,81.2,56.43,35.37,18.08;...
		437.3,380.5,327.7,279.3,232.9,192.8,154.3,120,90.02,63.54,40.37,20.47;...
		460.7,402.3,347.9,297.3,249.2,207.1,166.9,130.6,99.11,69.84,45.1,23.17;...
		484.3,423.7,367.8,315.6,265.3,221.4,179.6,141.2,107.6,76.82,49.52,25.7],[355.9,305.8,258,215.9,176.9,141.8,110.6,82.88,59.16,39.34,23.08,10.55;...
		379.8,327.1,278.9,235.1,192.8,157,123.3,93.13,68.04,46,27.96,13.31;...
		403.3,349.2,298.9,253.2,209.1,171.6,136.2,103.7,76.68,52.71,32.51,15.82;...
		426.8,371,319.2,271.2,225.5,185.9,148.4,114.7,85.59,59.39,37.07,18.19;...
		450.3,392.4,338.7,289.5,241.7,199.9,160.9,125.1,93.98,65.9,41.57,20.73;...
		473.6,414.1,358.9,306.9,257.5,214,173,135.8,102.5,72.33,46.1,23.11]),...
		cat(3,[352.1,302.4,255.1,213.4,174.9,140.2,109.2,82.23,58.93,39.33,23.83,11.54;...
		376.8,323.9,276.4,233.3,190.9,156,122.3,92.96,68.1,46.58,28.76,14.53;...
		401.2,346.6,296.3,251.2,207.9,170.6,135.5,104,77.14,53.48,33.39,17.14;...
		425.3,368.8,317.5,270.1,224.8,185.3,148.3,115.1,86.3,60.54,38.5,19.62;...
		449.1,390.8,337.4,288.1,241.2,199.9,160.8,125.5,94.88,67.22,43.45,22.35;...
		472.3,412.4,357.6,306.1,257.2,214.3,173.3,136.5,103.8,74.01,47.84,24.82],[343.4,294.1,247.8,206.4,168.2,134.5,104.3,77.55,55.01,36.28,21.23,9.75;...
		367.4,316.2,268.7,226,184.6,149.9,117.2,88.25,63.98,43.01,26.02,12.43;...
		391,337.9,289,244,201,164.5,130.2,98.82,72.71,49.86,30.54,15.04;...
		415.1,359.9,308.8,261.9,217.4,178.6,142.3,109.7,81.72,56.52,35.06,17.31;...
		438.3,381.6,328.4,279.9,234.1,193,155,120.5,90.31,63.09,39.63,19.79;...
		461.4,402.9,348.5,297.6,249.4,206.8,167,130.8,98.57,69.44,44.31,22.31])) ...
		,...
		cat(4,...
		cat(3,[71.25,65.86,59.62,53.91,47.94,42.3,36.27,29.95,23.92,17.68,11.03,4.16;...
		74.99,68.98,63.21,57.54,51.3,45.45,39.76,33.14,27.27,21.16,17.79,8.13;...
		78.3,72.21,66.35,60.62,54.53,48.81,42.86,36.59,30.6,24.5,18.08,11.41;...
		81.38,75.45,69.65,63.82,57.81,51.99,46.26,39.92,33.98,27.93,21.25,14.19;...
		84.68,78.45,72.58,67.16,60.8,55.38,49.13,42.92,37.24,30.65,24.04,17.24;...
		87.66,81.6,75.43,70.2,63.78,58.28,52.07,46.19,40.14,33.42,27.03,19.76],[67.95,62.62,56.6,50.6,45,39.39,33.48,27.57,21.58,15.57,9.28,3.04;...
		71.51,65.76,59.83,54.3,48.23,42.72,36.93,30.7,24.84,18.96,12.83,6.49;...
		74.74,68.99,63.13,57.41,51.43,45.88,40.02,33.93,28.16,22.18,16.01,9.52;...
		77.88,72.03,66.21,60.6,54.44,48.86,43.16,37.15,31.26,25.23,18.95,12.21;...
		80.98,74.99,69.29,63.55,57.57,52.11,46.03,40.07,34.37,28.2,21.72,14.91;...
		83.91,78.07,72.04,66.56,60.6,55.05,48.96,43.14,37.25,31.05,24.48,17.4]),...
		cat(3,[75.51,69.76,64.11,57.95,52.06,46.47,40.53,34.4,28.27,22.04,15.87,9.16;...
		78.57,73.11,66.85,61.34,55.26,49.57,43.76,37.48,31.48,25.54,18.88,12.45;...
		81.74,76.02,70.17,64.47,58.21,52.87,46.9,40.57,34.69,28.49,21.92,15.27;...
		84.82,79.23,73.47,67.77,61.59,55.81,49.97,43.62,37.83,31.56,24.97,17.8;...
		87.9,82.36,76.2,70.86,64.4,58.85,52.88,46.77,40.19,34.15,27.82,20.63;...
		91.11,85.2,79.28,73.65,67.45,61.74,55.65,49.8,43.58,37.19,30.5,23.02],[72.09,66.47,60.66,54.91,49.04,43.44,37.65,31.73,25.8,19.86,13.81,7.53;...
		75.35,69.48,63.77,58.17,52.23,46.59,40.93,34.99,29.01,22.98,16.74,10.5;...
		78.55,72.75,66.78,61.21,55.38,49.67,44.05,37.81,32,26.08,19.67,13.21;...
		81.4,75.71,69.98,64.27,58.31,52.67,47.09,40.86,35.08,28.83,22.54,15.68;...
		84.36,78.68,72.86,67.35,61.28,55.61,49.85,43.8,38.03,31.73,25.27,18.24;...
		87.24,81.72,75.88,70.32,64.24,58.57,52.67,46.87,40.84,34.56,27.87,20.57]),...
		cat(3,[74.62,68.91,63.32,57.2,51.15,45.53,39.83,33.64,27.42,21.12,14.88,8.07;...
		77.72,72.31,65.98,60.66,54.44,48.91,42.95,36.8,30.71,24.59,18.06,11.47;...
		80.87,75.22,69.4,63.68,57.54,52.22,46.09,39.85,33.87,27.75,21.07,14.35;...
		83.93,78.42,72.56,67.04,60.8,55.04,49.1,42.92,37.08,30.74,24.22,16.9;...
		87.13,81.56,75.48,70.2,63.54,58.05,52.06,45.98,39.99,33.63,26.94,19.73;...
		90.23,84.41,78.58,72.98,66.61,61.03,54.84,49.02,42.68,36.38,29.79,22.19],[71.36,65.63,59.85,54.1,48.23,42.7,36.84,31.02,24.99,19.02,12.98,6.5;...
		74.61,68.8,63,57.56,51.47,45.74,40.21,34.1,28.27,22.15,15.98,9.53;...
		77.65,71.95,65.97,60.49,54.54,48.96,43.25,37.15,31.3,25.21,18.78,12.27;...
		80.6,75.02,69.13,63.59,57.55,51.93,46.3,40.02,34.3,28.11,21.67,14.76;...
		83.55,77.86,72.08,66.59,60.46,54.68,49.06,42.95,37.21,30.85,24.37,17.39;...
		86.49,80.86,74.91,69.53,63.4,57.85,51.9,45.97,39.99,33.67,27.05,19.63]),...
		cat(3,[78.42,72.5,67.05,61.27,55.14,49.32,43.61,37.86,31.79,25.42,19.22,12.39;...
		81.85,75.77,70.2,64.53,58.08,52.62,46.97,40.89,34.7,28.72,22.16,15.44;...
		84.73,78.97,73.82,67.67,61.22,55.83,50.1,43.72,37.85,31.68,24.88,18.08;...
		87.82,81.99,76.15,70.71,64.42,59.01,53.08,46.84,40.98,34.65,27.8,20.47;...
		90.74,85.08,78.87,73.62,67.45,62,55.83,49.76,43.75,37.44,30.55,23.17;...
		93.75,87.98,81.89,76.51,70.48,64.61,58.78,52.63,46.66,40.12,33.26,25.7],[75.02,69.45,63.6,58.09,52.08,46.54,40.76,35.04,29.13,27.1,17.18,10.55;...
		78.09,72.72,66.78,61.42,55.25,49.7,44.01,37.92,32.12,26.1,19.79,13.31;...
		81.31,75.59,69.9,64.28,58.18,52.69,47.08,40.94,35.04,29,22.53,15.82;...
		84.34,78.45,72.94,67.43,61.19,55.51,49.78,43.92,38.04,31.89,25.28,18.19;...
		87.34,81.52,75.64,70.3,63.99,58.58,52.73,46.74,41.01,34.66,27.86,20.73;...
		90.08,84.23,78.53,73.07,67.02,61.37,55.62,49.59,43.66,37.28,30.54,23.11]),...
		cat(3,[77.73,71.84,66.17,60.48,54.17,48.57,42.67,37.07,31,24.35,18.33,11.54;...
		81,75.01,69.38,63.8,57.38,51.97,46.11,40,33.97,27.96,21.21,14.53;...
		83.98,77.97,72.46,66.96,60.37,54.94,49.29,42.94,37.08,30.92,24.18,17.14;...
		87.05,81.18,75.49,69.81,63.62,58.2,52.24,46.03,40.06,33.81,26.95,19.62;...
		90.01,84.29,78.09,72.8,66.67,61.3,54.97,49.03,43.03,36.59,29.74,22.35;...
		92.88,87.25,81.04,75.7,69.6,63.86,58,51.83,45.77,39.28,32.53,24.82],[74.25,68.56,62.8,57.37,51.26,45.75,39.9,34.16,28.32,22.26,16.28,9.75;...
		77.27,71.83,66.07,60.54,54.46,48.97,43.22,37.13,31.31,25.28,18.98,12.43;...
		80.43,74.79,69.17,63.5,57.43,52.03,46.3,40.16,34.25,28.28,21.69,15.04;...
		83.5,77.61,72.11,66.62,60.32,54.75,49.07,43.12,37.33,31.11,24.47,17.31;...
		86.53,80.64,74.91,69.43,63.16,57.71,51.94,46.01,40.13,33.85,27.06,19.79;...
		89.28,83.42,77.79,72.3,66.24,60.6,54.75,48.89,42.86,36.41,29.6,22.31])) ...
		);
		
		vCritsRequested = (vCritArray(vKplus1,vN_r,vSigni,vCase,vStat));
                